<?php

namespace HCM\FPA\Calculos\Ferias;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\GpesitufuncmEntity; 
use HCM\FPA\Entity\FpagozoferiasEntity;
use HCM\FPA\Entity\FpaferiasEntity;
use HCM\FPA\Calculos\Geral\Inicializa;
use HCM\FPA\Entity\FpafaltaferiasEntity;
use HCM\FPA\Entity\FpatipoferiasmEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaassoceventoEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;

class FeriasColetiva{

    protected $_parametros;
    protected $_nrOrgPadrao;
    protected $_nrOrgTrabalho;
    protected $_mesCompetencia;
    protected $_dtPagamento;
    protected $_ultPagamento;
    protected $dtInicio;
    protected $dtFim;
    protected $_dsInicio;
    protected $_dsFim;
    protected $_id_retrocesso;
    protected $_id_fazDifComp;
    protected $_dsOrdem;
    protected $_vinculom;
    protected $_fpaGozoFerias;
    protected $_fpaFerias;
    protected $_nrTipoModalidadeCalculo;
    protected $_nrOcorrenciaCalculo;
    protected $vinculo;
    protected $_jornada_diaria; 
    protected $_eventom;
    protected $_escalam;
    protected $escala;
    protected $_faltas_intervalo;
    protected $diasDireito;
    protected $diasJaGozados;
    protected $diasGozo;
    protected $diasLicenRemun;
    protected $dtIniLicen;
    protected $dtFimLicen;
    protected $_situFuncm;
    protected $falta_ferias;
    
    public function __construct($entityManager, $hcmParameters, $dataSet){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->dataSet = $dataSet;
        $this->_parametros = $hcmParameters;
        $this->_nrOrgTrabalho = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->_mesCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia());
        $this->_buscaDadosFormularioFerias($dataSet);
        
        Inicializa::Container($entityManager, $hcmParameters, $this->_dtPagamento);
        Inicializa::carregaVariaveis();
        $this->_vinculom = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_fpaGozoFerias = new FpagozoferiasEntity($entityManager, $hcmParameters);
        $this->_fpaFerias = new FpaferiasEntity($entityManager, $hcmParameters);
        $this->_nrTipoModalidadeCalculo = $this->hcmParameters->getTipoModalidadeCalculo();
        $this->_nrOcorrenciaCalc = $hcmParameters->getNrOcorrenciaCalculo();
        $this->_eventom = TkDI::getContainer()->eventom;
        $this->_escalam = TkDI::getContainer()->escalam;
        $this->_alteracao_situacao_funcional = TkDI::getContainer()->alteracao_situacao_funcional;
        $this->_faltas_intervalo = TkDI::getContainer()->faltas_intervalo;
        $this->_falta_ferias = new FpafaltaferiasEntity($entityManager, $hcmParameters);
        
    }

    public function preparaColetiva(){
        try {
            $msg = null;
            
            $vinculos = $this->_vinculom->retornaVinculosPorTipoCalculo(2 , $this->_nrOrgTrabalho, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim,
                $this->dtInicio, $this->dtFim, $this->_dsOrdem, null, $this->dsvincnaocontempl, true);
            
            $dtFormInicio = $this->dtInicio;
            $dtFormFim = $this->dtFim;
            
            if ($vinculos) {
                
                foreach($vinculos as $vinculo){
                    
                    $this->dtInicio = $dtFormInicio;
                    $this->dtFim = $dtFormFim;
                    
                    $this->_alteracao_situacao_funcional->retornaAlteracaoSituacaoFuncional(
                       $vinculo->getNrorg(), $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc()
                    );
                    
                    MsgCalculo::getInstance()->setDados(null, $vinculo->getNrvinculom(), $this->_nrOrgTrabalho, 'FERIASCOLETIVA');
                    
                    if($this->dtInicio < $this->_mesCompetencia){
                        MsgCalculo::getInstance()->adicionaMensagem('Data de inicio do Gozo deve ser posterior a competência atual', MsgCalculo::ERRO);
                        
                        MsgCalculo::getInstance()->salvaMensagens();
                        MsgCalculo::getInstance()->clear();
                        continue;
                    }
                    
                    //Bloqueia adicionar gozo de ferias se o vinculo ja possuir cadastro de gozo entre este periodo.
                    $gozoFeriasJaCadastrado = $this->_fpaGozoFerias->retornaGozosFeriasEntreDatas($this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), $this->dtInicio, $this->dtFim);
                    
                    if ($gozoFeriasJaCadastrado) {
                        MsgCalculo::getInstance()->adicionaMensagem("Operação Bloqueada!! O vínculo ".$vinculo->getNrvinculom()." ".$vinculo->getNmvinculom()." Já possui um período de gozo de férias que contempla um ou mais dias deste período!", MsgCalculo::ERRO);
                        
                        MsgCalculo::getInstance()->salvaMensagens();
                        MsgCalculo::getInstance()->clear();
                        continue;
                    }
                        
                    if(!$this->_fpaGozoFerias->validaPersisteGozoFerias($vinculo, $this->dtInicio, $this->dtFim)){
                        $msgs = $this->_fpaGozoFerias->getMensagensGozoInvalido();
                        //incluir na msg de calculo
                        foreach ($msgs as $msg) {
                            MsgCalculo::getInstance()->adicionaMensagem($msg['msg'], MsgCalculo::ERRO);
                        }
                            
                        MsgCalculo::getInstance()->salvaMensagens();
                        MsgCalculo::getInstance()->clear();
                        continue;
                    }
                    
                    $this->vinculo = $vinculo;
                    $this->menor1Ano = DateUtil::getQtdeDiasEntre($vinculo->getDtadmissaovinc(), $this->dtInicio) < 365 ? true : false;
                    $this->diasLicenRemun = 0;
                    $persisteGozo = true;
                    
                    // Retorna Escala de Trabalho
                    $nrEscala = $vinculo->historico->getNrescalatrabm();
                    if (!$nrEscala) {
                        MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem escala de trabalho.');
                        continue;
                    }
                    
                    $this->escala = $this->_escalam->retornaEscalaNrEscala($nrEscala, $this->_nrOrgTrabalho);
                    
                    if ($this->escala->historico) {
                        $this->_jornada_diaria = $this->escala->historico->getQthrescalatrabh();
                    }
                    
                    $ferias = $this->preparaFerias(); 
                    
                    if($ferias){
                        $this->diasGozo = DateUtil::getQtdeDiasEntre($this->dtInicio, $this->dtFim) + 1;
                        
                        //busca valores de referencia.
                        $Art140 = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'ART_140_FER_COLET');
                        $fechaPeriodoMaior1Ano = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'FECHA_PERIODO_MAIOR_1ANO');
                        $zeraMenorQueDias = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'ZERA_DIAS_DIREITO_MEN');
                        $reduzGozoLicenca = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'REDUZ_GOZO_LICENCA');
                        $fechaReduzPeriodo = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'FECHA_PERIODO_REDUZINDO');
                        
                        //Reduz gozo para sobrar a quantidade permitida pela lei para se conseder a proxima ferias e paga o resto das ferias coletivas como licensa.
                        if($reduzGozoLicenca){
                            $diasDireitoReduz = $this->faltaFerias->getQtdiasmaxfe() - $reduzGozoLicenca;
                            if(($this->diasGozo + $this->diasJaGozados) <= $this->faltaFerias->getQtdiasmaxfe() && ($this->diasGozo + $this->diasJaGozados) > $diasDireitoReduz){
                                $this->diasGozo -= ($this->diasGozo + $this->diasJaGozados) - $diasDireitoReduz;
                                $this->dtFim = DateUtil::adicionaIntervalo($this->dtInicio, $this->diasGozo - 1);
                            }
                        }
                        
                        if ($zeraMenorQueDias && $this->diasDireito < $zeraMenorQueDias || $this->diasGozo < $zeraMenorQueDias) {
                            $this->diasDireito = 0;
                            $this->diasGozo = 0;
                            $persisteGozo = false;
                            MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$vinculo->getNmvinculom().' sem dias de direito superior a '.$zeraMenorQueDias);
                        }
                        //Fecha Periodo Reduzido
                        if($fechaReduzPeriodo && $ferias->getDtfimaquisicao() > $this->dtInicio && $this->diasGozo < $this->diasDireito
                        && $this->menor1Ano){
                            
                            $dtIniAquisicao = DateUtil::subtraiIntervalo($ferias->getDtiniaquisicao(), 1);
                        
                            $mesesDireito = round(($this->diasGozo + $this->diasJaGozados) / 2.5);
                            
                            $data_fim_per = DateUtil::adicionaIntervalo($ferias->getDtiniaquisicao(), $mesesDireito, DateUtil::MES);
                            
                            $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL);
                            $ferias->setDtfimaquisicao(DateUtil::subtraiIntervalo($data_fim_per, 1));
                            $databaseferias = DateUtil::adicionaIntervalo($ferias->getDtfimaquisicao(), 1);
                            $vinculo->historico->atualizaHistorico($this->_mesCompetencia, $databaseferias);
                        }else{
                            if(($Art140 && $this->menor1Ano && $persisteGozo) || ($fechaPeriodoMaior1Ano && !$this->menor1Ano && $this->diasGozo > $this->diasDireito) 
                                || ($zeraMenorQueDias && $this->diasDireito > $zeraMenorQueDias && $this->menor1Ano) 
                            ) {
                                $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL);
                                $ferias->setDtfimaquisicao(DateUtil::subtraiIntervalo($this->dtInicio, 1));
                            }
                        }
                        
                        $this->setaDiasLicenca();
                        
                        //Inclui licença remunerada e situaçao normal após a licença.
                        if($this->nrSituFuncm && $this->nrSituFuncm != GpesitufuncmEntity::SITUACAO_FUNCIONAL_NORMAL && $this->diasLicenRemun > 0) {
                            
                            $this->_alteracao_situacao_funcional->incluiAltSituFunc(
                                $vinculo->getNrvinculom(), $vinculo->getNrorg(), GpesitufuncmEntity::SITUACAO_FUNCIONAL_LICENCA_REMUNERADA, $this->dtIniLicen, $this->dtFimLicen
                            );

                            $this->_alteracao_situacao_funcional->incluiAltSituFunc(
                                $vinculo->getNrvinculom(), $vinculo->getNrorg(), GpesitufuncmEntity::SITUACAO_FUNCIONAL_NORMAL,  DateUtil::adicionaIntervalo($this->dtFimLicen, 1), null
                            );
                        }
                        
                        //Altera Data Base
                        if ($this->diasDireito >= $zeraMenorQueDias && $persisteGozo){
                            $databaseferias = DateUtil::adicionaIntervalo($ferias->getDtfimaquisicao(), 1);
                            $vinculo->historico->atualizaHistorico($this->_mesCompetencia, $databaseferias);
                        }
                        
                        $diasFeriasSobrando = $this->diasDireito - $this->diasGozo >= 0 ? $this->diasDireito - $this->diasGozo : $this->diasDireito;
                        
                        $ferias->setQtdiasmaxfe($diasFeriasSobrando);
                        
                        if($this->faltaFerias->getQtdiasmaxfe() == $this->diasJaGozados + $this->diasGozo){
                            $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL);
                        }
                        
                        if($persisteGozo){
                            
                            if($ferias->getIdcontrolferias() == 'ABERTO' || $ferias->getIdcontrolferias() == 'ADQUIRIDO'){
                                $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL);
                            }
                            
                            $this->_fpaGozoFerias->incluiGozoFerias($vinculo->getNrorg(), $ferias->getNrferias(), $this->dtInicio, $this->dtFim, 0,
                            DateUtil::adicionaIntervalo($this->dtFim, 1), 'N', $this->dsFeriasColet);
                        }
                        
                        $this->entityManager->persist($ferias);
                        $this->entityManager->flush($ferias);
                    }else{
                        MsgCalculo::getInstance()->adicionaMensagem('Período de Férias não encontrado');
                        continue;
                    }
                    
                    MsgCalculo::getInstance()->salvaMensagens();
                    MsgCalculo::getInstance()->clear();
                }
                
            }else{
                $msg = 'Nenhum vinculo encontrado';
            }
            
            return $msg;
        
        }catch(\Exception $e){
            throw new \Exception($e);
        }    
        
    }
    
    public function cancelaColetiva(){
        try {
            
            $vinculos = $this->_vinculom->retornaVinculosPorTipoCalculo(2 , $this->_nrOrgTrabalho, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim,
                $this->dtInicio, $this->dtFim, $this->_dsOrdem, null, $this->dsvincnaocontempl);
                
            $confirmacaoLiquidoVinculos = $this->dataSet['CONFIRMACAOLIQUIDOFERIAS'];
            
            if ($vinculos) {
                
                if (!$confirmacaoLiquidoVinculos) {
                    $vinculosComLiquido = $this->verificaExistenciaLiquidoFeriasVinculos($vinculos);
                    if($vinculosComLiquido){
                        $textoVinculosComLiquido = $this->montaMensagemVinculosLiquido($vinculosComLiquido);
                        return array('MENSAGEM' => Messages::HCM01000_FPA_CANCELAMENTO_FERIAS_EXISTEM_VINCULOS_COM_LIQUIDO_FERIAS, 'VINCULOS' => $textoVinculosComLiquido);
                    }
                }
                
                foreach($vinculos as $vinculo){
                    $dataBase = null;
                    $ferias = null;
                    $this->vinculo = $vinculo;
                    MsgCalculo::getInstance()->setDados(null, $vinculo->getNrvinculom(), $this->_nrOrgTrabalho, 'ERROSCALCULO');
                    
                    $this->_fpaFerias->retornaFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());

                    $this->_fpaGozoFerias->retornaGozoFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());
                    
                    $itemGozo = $this->_fpaGozoFerias->procuraGozoFerias($this->dtInicio);
                    
                    if($itemGozo){
                        $ferias = $this->_fpaFerias->procuraFerias($itemGozo->getNrferias());
                    }else{
                        $this->retrocedeLicensas($this->dtInicio);
                        MsgCalculo::getInstance()->salvaMensagens();
                        MsgCalculo::getInstance()->clear();
                        $this->entityManager->flush();
                        continue;
                    }
                    
                    $this->menor1Ano = DateUtil::getQtdeDiasEntre($vinculo->getDtadmissaovinc(), $this->dtInicio) < 365 ? true : false;
                    $this->diasLicenRemun = 0;
                    
                    // Retorna Escala de Trabalho
                    $nrEscala = $vinculo->historico->getNrescalatrabm();
                    if (!$nrEscala) {
                        MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem escala de trabalho.');
                        continue;
                    }
                    $this->escala = $this->_escalam->retornaEscalaNrEscala($nrEscala, $this->_nrOrgTrabalho);
                    
                    if ($this->escala->historico) {
                        $this->_jornada_diaria = $this->escala->historico->getQthrescalatrabh();
                    }
                    
                    if($ferias) {
                        
                        $this->diasGozo = DateUtil::getQtdeDiasEntre($this->dtInicio, $this->dtFim) + 1;
                        
                        //busca valores de referencia.
                        $Art140 = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'ART_140_FER_COLET');
                        $fechaPeriodoMaior1Ano = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'FECHA_PERIODO_MAIOR_1ANO');
                        $zeraMenorQueDias = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia, 'ZERA_DIAS_DIREITO_MEN');
                        
                        $this->retrocedeLicensas(DateUtil::adicionaIntervalo($itemGozo->getDtfimgozoferias(), 1));
                        
                        $dtFimAquisitivo = DateUtil::subtraiIntervalo(DateUtil::adicionaIntervalo($ferias->getDtiniaquisicao(), 1, DateUtil::ANO), 1);
                        $ferias->setDtfimaquisicao($dtFimAquisitivo);
                        
                        if ($dtFimAquisitivo < $this->_mesCompetencia) {
                            $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO);
                        } else {
                            $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_ABERTO);
                        }
                        
                        if ($this->menor1Ano) {
                            $dataBase = $vinculo->getDtadmissaovinc();
                        }
                        
                        if($fechaPeriodoMaior1Ano && $ferias->getIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_ABERTO)){
                            $dataBase = DateUtil::adicionaIntervalo($ferias->getDtfimaquisicao(), 1);
                        }
                        
                        if($dataBase){
                            $vinculo->historico->atualizaHistorico($this->_mesCompetencia, $dataBase);
                        }
                        
                        $itemGozoAnterior = $this->_fpaGozoFerias->procuraGozoFeriasAnterior($this->dtInicio, $ferias->getNrferias());
                        if (is_object($itemGozoAnterior)) {
                            $ferias->setIdcontrolferias(FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL);
                        }
                        
                        $this->_fpaGozoFerias->deletaGozoFerias($itemGozo->getNrgozoferias(), $itemGozo->getNrorg(), $itemGozo->getNrferias());
                        
                        $this->entityManager->persist($ferias);
                        $this->entityManager->flush($ferias);
                        MsgCalculo::getInstance()->salvaMensagens();
                        MsgCalculo::getInstance()->clear();
                        
                    }
                }
            }else{
                throw new \Exception('Nenhum vinculo encontrado');
            }
            
        }catch(\Exception $e){
            throw new \Exception($e);
        }
    }
    
    public function retrocedeLicensas($dtFim){
        $this->_alteracao_situacao_funcional->retornaAlteracaoSituacaoFuncional(
            $this->vinculo->getNrorg(), $this->vinculo->getNrvinculom(), $this->vinculo->getDtadmissaovinc()
        );

        $itemsAltSituFuncLic = $this->_alteracao_situacao_funcional->procuraAltSituFunc($dtFim, 17
        );
        
        if ($itemsAltSituFuncLic) {
            
            foreach ($itemsAltSituFuncLic as $itemAltSituFuncLic) {                                // 
                $dt_situ_normal = DateUtil::adicionaIntervalo($itemAltSituFuncLic->getDtfimsitufunc(), 1);

                $this->_alteracao_situacao_funcional->deletaAltSituFunc($itemAltSituFuncLic);
              
                $itemsAltSituFuncNorm = $this->_alteracao_situacao_funcional->procuraAltSituFunc(
                    $dt_situ_normal, 1
                );
                
                foreach ($itemsAltSituFuncNorm as $itemAltSituFuncNorm) {
                    if ($itemAltSituFuncNorm) {
                        $this->_alteracao_situacao_funcional->deletaAltSituFunc($itemAltSituFuncNorm);
                    }
                }
                
            }
        }
    }
    
    public function setaDiasLicenca(){
        if ($this->diasGozo > $this->diasDireito) {
            if ($this->diasDireito != 0) {
                    $this->diasLicenRemun = $this->diasGozo - $this->diasDireito;
                    $this->dtFim = DateUtil::subtraiIntervalo($this->dtFim, $this->diasLicenRemun);
                    $this->dtIniLicen = DateUtil::adicionaIntervalo($this->dtFim, 1);
                    $this->dtFimLicen = DateUtil::adicionaIntervalo($this->dtIniLicen, $this->diasLicenRemun - 1);
                    if($this->diasGozo > $this->diasDireito){
                        $this->diasGozo = $this->diasDireito;
                    }
            } else {
                $this->diasLicenRemun = $this->diasGozo;
                $this->dtIniLicen = $this->dtInicio;
                $this->dtFimLicen = $this->dtFim;
            }
        }
    }
    
    public function verificaExistenciaLiquidoFeriasVinculos($vinculos){
        $assoc_eventos = new FpaassoceventoEntity($this->entityManager, $this->hcmParameters);
        $eventom = new FpaeventomEntity($this->entityManager, $this->hcmParameters);
        $fpacalculofolhaEntity = new FpacalculofolhaEntity($this->entityManager, $this->hcmParameters);
        
        $vinculosComLiquido = array();
        
        foreach ($vinculos as $vinculo){
            
             //Verifica liquido
            $calc_folhaAtual = $fpacalculofolhaEntity->retornaCalculoFolha(
                $this->_nrOrgTrabalho, FpatipomovimentEntity::TP_MOV_FERIAS, $this->_mesCompetencia, 1, 1
            )[0];
            
            $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
            $itemCalcCompetenciaAtual->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAtual->getNrcalculofolha());
            $liquido = $itemCalcCompetenciaAtual->verificaLiquido(
                $assoc_eventos, $eventom, $this->_nrOrgTrabalho
            );
            
            if ($liquido) {
                array_push($vinculosComLiquido, $vinculo);
            }
            
        }
        
        return !empty($vinculosComLiquido) ? $vinculosComLiquido : false;
    }
    
    public function preparaFerias(){
          
        $this->_fpaFerias->retornaFerias($this->vinculo->getNrorg(), $this->vinculo->getNrvinculom());
        
        $item = $this->_fpaFerias->procuraFerias();
        
        if($item){
            
            $diasFaltas = $this->_fpaFerias->_totalizaDiasFaltas(
                $this->vinculo, $this->_eventom, $this->_faltas_intervalo, DateUtil::setDay($item->getDtiniaquisicao(), 1),
                $item->getDtfimaquisicao(), $this->_jornada_diaria, $this->_nrTipoModalidadeCalculo,
                $this->_nrOcorrenciaCalc
            );
            
            $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
            
            if($falta_ferias){
                $this->faltaFerias = $falta_ferias;
                $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                $qtd_dias = $falta_ferias->getQtdiasmaxfe();
            }else{
                MsgCalculo::getInstance()->adicionaMensagem('Falta ferias não parametrizado para o tipo de ferias '.$item->getNrtipoferias());
                return false;
            }
            
            $qtd_doze_avos = 0;
            $this->diasJaGozados = 0;
            if($item->getIdcontrolferias() == FpaferiasEntity::IDCONTROLFERIAS_ABERTO){
                
                $dtIniAquisicao = DateUtil::subtraiIntervalo($item->getDtiniaquisicao(), 1);
               
                $dif = strtotime($this->dtInicio->format('Y/m/d')) - strtotime($dtIniAquisicao->format('Y/m/d'));
 
                $qtd_doze_avos = floor($dif / (60 * 60 * 24 * 30)) > 12 ? 12 : floor($dif / (60 * 60 * 24 * 30));
                
                $data_fim_per = DateUtil::adicionaIntervalo($item->getDtiniaquisicao(), $qtd_doze_avos, DateUtil::MES);
                
                $dtIniGozoAux = DateUtil::subtraiIntervalo(substr($this->dtInicio, 0, 10), 1);
                
                if ($data_fim_per->format('d/m/Y') == $dtIniGozoAux->format('d/m/Y')
                    || $data_fim_per < $dtIniGozoAux
                ) {
                    $dias = DateUtil::getQtdeDiasEntre($data_fim_per, $dtIniGozoAux) + 1;
                    if ($dias >= 15) {
                        $qtd_doze_avos++;
                    }
                }
                
                $nrTipoFerias = $item->getNrtipoferias();
                $mult = FpatipoferiasmEntity::retornaDiasDireito($nrTipoFerias);
                
                $mes_afastados = $this->_fpaFerias->_quantidadeMesAfastados($dtIniAquisicao, $data_fim_per);
                
                $qtd_doze_avos -= $mes_afastados;
                
                $this->diasDireito = ceil($qtd_doze_avos * $mult);
                
                return $item;
                
            }
            
            if($item->getIdcontrolferias() == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL){
                foreach ($this->_fpaFerias->getGozoFerias($item->getNrferias()) as $itemGozo) {
                    $this->diasJaGozados += $itemGozo->getDiasGozados() + $itemGozo->getDiasAbonados();
                }
            }
            
            $mes_afastados = $this->_fpaFerias->_quantidadeMesAfastados($item->getDtiniaquisicao(), $item->getDtfimaquisicao());
            $dias_afastados = $mes_afastados > 0 ? $mes_afastados * 2.5 : 0;
            
            $this->diasDireito = $qtd_dias - $this->diasJaGozados - $dias_afastados;
            
            return $item;
        }
        
        return false;
    }
    
    public function _buscaDadosFormularioFerias($dataSet){
        
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->dtInicio = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOINI'],0, 10));
        $this->dtFim = DateUtil::getDataDeString(substr($dataSet['DTINIGOZOFIM'], 0, 10));
        $this->dtRetorno = $dataSet['DTRETORNOFERIAS'];
        $this->dsFeriasColet = $dataSet['DSFERIASCOLET'];
        $this->nrSituFuncm = array_key_exists('NRSITUFUNCM', $dataSet) ? $dataSet['NRSITUFUNCM'] : null;
        $this->dsFeriasColet = $dataSet['DSFERIASCOLET'];
        $this->dsvincnaocontempl = array_key_exists('DSVINCNAOCONTEMPL', $dataSet) && isset($dataSet['DSVINCNAOCONTEMPL']) ? implode(',', $dataSet['DSVINCNAOCONTEMPL']) : null;
    }
  
    public function finalizaMensagens() {
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ERRO;
        } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $this->_msg_idstatus != MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ALERTA;
        }
        if ($msgcalculo['IDABORT'] === true) {
            $this->_msg_idabort = true;
            $this->_msg_mensagens = $msgcalculo['MENSAGENS'];
            return false;
        } else {
            MsgCalculo::getInstance()->salvaMensagens();
        }
        MsgCalculo::getInstance()->clear();
        return true;
    }
    
    public function montaMensagemVinculosLiquido($vinculos){
         $textoVinculosComLiquido = '';
        foreach ($vinculos as $vinculo){
            $nomeVinculo = $vinculo->getNmvinculom();
            $numeroVinculo = $vinculo->getNrvinculom();
            $textoVinculosComLiquido .= 'Nome: ' . $nomeVinculo . ' | Número: ' . $numeroVinculo . ', <br />';
        }
        $textoVinculosComLiquido = substr($textoVinculosComLiquido, 0 , -8);
        return $textoVinculosComLiquido;
    }
}