<?php

namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\registro;

use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\RegistroHolerite;
use HCM\Util\DateUtil;

class Registro2HoleriteSantander extends RegistroHolerite{

    const REGISTRO_2 = '2';

    
    private $_pessoa;
    
    private $_vinculo;
    
    private $_itemCalcVinculo;

    private $_sequencial;

    public function geraRegistro2(){
        $this->_geraRegistro();
    }

    private function _geraRegistro(){
        // Bloco 01 | 0001-0001 | Tipo de registro = 2.
        $this->atribuiDado(self::REGISTRO_2, 1);

        // Bloco 02 | 0002-0021 | Matrícula do funcionário.
        $this->atribuiDado($this->_vinculo->getCdmatricula(), 20, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 03 | 0022-0024 | Número de Sequência do Lançamento.
        $this->atribuiDado((string)$this->_sequencial, 3, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 04 | 0025-0031 | Código do lançamento.
        $this->atribuiDado($this->_itemCalcVinculo->getNritemcalcfolha(), 7, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 05 | 0032-0054 | Descrição do lançamento.
        // @todo Holerite Colocar maiusculo
        $this->atribuiDado($this->_itemCalcVinculo->evento->historico->getNmeventoh(), 23, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 06 | 0055-0057 | Livre (Preencher com espaços).
        $this->atribuiDado(self::CARACTER_BRANCO, 3, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 07 | 0058-0066 | Valor do lançamento.
        $this->atribuiDado(sprintf('%1.2f', $this->_itemCalcVinculo->getVreventofolha()), 9, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 08 | 0067-0146 | Lançamento (Internet Banking).
        // @todo Holerite Colocar maiusculo
        $this->atribuiDado($this->_itemCalcVinculo->evento->historico->getNmeventoh(), 80, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 09 | 00147-0147 | Tipo de lançamento (Débito = 'D' / Crédito = 'C').
        $this->atribuiTipoLancamento($this->_itemCalcVinculo->evento->getIdtipoevento());

        // Bloco 10 | 00148-0156 | Unidade de trabalho.
        // @todo Holerite Colocar maiusculo
        $this->atribuiDado($this->_itemCalcVinculo->evento->historico->getIdunidrefereven(), 9, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 11 | 0157-0161 | Quantidade de horas trabalhadas.
        $this->atribuiDado(strval($this->_itemCalcVinculo->getVrreffolha()), 5, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);
        
        // Bloco 12 | 0162-0162 | Livre (Preencher com espaços).
        $this->atribuiDado(self::CARACTER_BRANCO, 1, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);

        // Bloco 06 | 0163-0170 | Uso privativo do banco (Preencher com 00010101).
        $this->atribuiDado('00010101', 8, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 06 | 0171-0178 | Uso privativo do banco (Preencher com 00010101).
        $this->atribuiDado('00010101', 8, self::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 06 | 0179-0400 | Livre (Preencher com espaços).
        $this->atribuiDado(self::CARACTER_BRANCO, 222, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);
    }


    public function setPessoa($pessoa)
    {
        $this->_pessoa = $pessoa;
    }


    public function setVinculo($vinculo)
    {
        $this->_vinculo = $vinculo;
    }

    public function setItemCalculo($itemCalculo)
    {
        $this->_itemCalcVinculo = $itemCalculo;
    }

    public function setSequencial($sequencial)
    {
        $this->_sequencial = $sequencial;
    }


    private function atribuiTipoLancamento($idtipoevento){

        switch ($idtipoevento) {
            case 'PROVENTO' :
                $idTipoLanc = self::TIPO_LANC_CREDITO;
                break;
            case 'DESCONTO' :
                $idTipoLanc = self::TIPO_LANC_DEBITO;
                break;
            default :
                $idTipoLanc = ' ';
                break;
        }

        $this->atribuiDado($idTipoLanc, 1, self::CARACTER_BRANCO, self::PREENCHE_DIREITA);
    }

}