<?php

namespace HCM\PTO\Logic\AFD\Portaria671;

/**
 * Portaria 671
 * Cabecalho do arquivo AFD
 * 
 */

use HCM\PTO\Logic\AFD\Cabecalho as CabecalhoAFD;
use HCM\PTO\Logic\AFD\{ Parametros, RegistroPorConsulta };

class Cabecalho extends RegistroPorConsulta implements CabecalhoAFD {
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Parametros $parametros) {
        parent::__construct($entityManager, $parametros, new ValidadorRegistroCRC16);
    }
    
    /**
     * Portaria 671
     * Retorna a query a ser utilizada pelo registro de cabecalho
     * @return string
     */
    public function obtemConsultaRegistro(): string {
        return <<<SQL
            SELECT
              SUBSTR(
                  /* 001-009 "000000000" */ '000000000' ||
                  /* 010-010 Tipo do registro. Preencher com "1" */ '1' ||
                  /* 011-011 Tipo de identificador do empregador: - "1": CNPJ; - "2": CPF */ '1' ||
                  /* 012-025 CNPJ ou CPF do empregador */ LPAD(NVL(CPNJ, ' '), 14, ' ') ||
                  /* 026-039 CNO (Cadastro Nacional de Obras) ou CAEPF (Cadastro de Atividade Econômica da Pessoa Física), quando existir. */ LPAD(NVL(CAEPF, ' '), 14, ' ') ||
                  /* 040-189 Razão social ou nome do empregador */ RPAD(RAZAO_SOCIAL, 150) ||
                  /* 190-206 Número de registro no INPI, no caso de REP-P */ LPAD(NVL(INPI, ' '), 17, ' ') ||
                  /* 207-216 Data inicial dos registros no arquivo. */ RPAD(NVL(TO_CHAR(NVL(TO_DATE(:P_INICIO, 'DD/MM/YYYY'), DTINICIALAPURACAO), 'YYYY-MM-DD'), ' '), 10, ' ') ||
                  /* 217-226 Data final dos registros no arquivo. */ RPAD(NVL(TO_CHAR(NVL(TO_DATE(:P_FIM, 'DD/MM/YYYY'), DTFINALAPURACAO), 'YYYY-MM-DD'), ' '), 10, ' ') ||
                  /* 227-250 Data e hora da geração do arquivo. */ TO_CHAR(SYSDATE, 'YYYY-MM-DD') || 'T' || TO_CHAR(SYSDATE, 'HH24:MI') || ':00' || TO_CHAR(FROM_TZ(CAST (SYSDATE AS TIMESTAMP), SESSIONTIMEZONE),'TZHTZM') ||
                  /* 251-253 Versão do leiaute do AFD. Preencher com "003" */ '003' ||
                  /* 254-254 Tipo de identificador do fabricante ou desenvolvedor do REP: - "1": CNPJ; - "2": CPF */ '1' ||
                  /* 255-268 CNPJ ou CPF do fabricante ou desenvolvedor do REP */ '14288794000145' ||
                  /* 269-298 Modelo, no caso de REP-C */ LPAD(' ', 30, ' ')
                  /* 299-302 CRC-16 do registro. Gerado posteriormente */
              , 0, 298) AS AFD
            FROM (
                SELECT
                  EH.NMRAZSOCESTRUT AS RAZAO_SOCIAL,
                  PA.DTINICIALAPURACAO,
                  PA.DTFINALAPURACAO,
                  REPLACE(TRANSLATE( EH.CDCNPJESTRUT, TRANSLATE('0' || EH.CDCNPJESTRUT, '0123456789', '@' ), '@' ), '@', '' ) CPNJ,
                  REPLACE(TRANSLATE( EH.CDCAEPFESTRUT, TRANSLATE('0' || EH.CDCAEPFESTRUT, '0123456789', '@' ), '@' ), '@', '' ) CAEPF,
                  REPLACE(TRANSLATE( VRH.VRREFERENCIA, TRANSLATE('0' || VRH.VRREFERENCIA, '0123456789', '@' ), '@' ), '@', '' ) INPI
                FROM ESTRUTURAM EM
                LEFT JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (
                      SELECT MAX(DTMESCOMPETENC)
                      FROM ESTRUTURAH
                      WHERE NRORG = EH.NRORG
                          AND NRESTRUTURAM = EH.NRESTRUTURAM
                          AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
                LEFT JOIN FPA_VALORREFERENCIAM VRM ON VRM.NRORG IN ( :P_NRORG_PADRAO, EM.NRORG )
                    AND VRM.DSREFERENCIA = 'INPI'
                    AND VRM.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM FPA_VALORREFERENCIAM
                                               WHERE NRORG          = VRM.NRORG
                                                 AND DSREFERENCIA   = VRM.DSREFERENCIA
                                                 AND DTINIVIGENCIA <= :P_DTMESCOMPETENC )
                LEFT JOIN FPA_VALORREFERENCIAH VRH ON VRH.NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                    AND VRH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_VALORREFERENCIAH
                                                WHERE NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                                                  AND DTMESCOMPETENC    <= :P_DTMESCOMPETENC )
                LEFT JOIN PTO_PERIODOAPURACAO PA ON PA.NRORG = EM.NRORG
                    AND PA.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
                WHERE EM.NRORG = :P_NRORG
                    AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
            )
SQL;
    }
    
    /**
     * Portaria 671
     * Retorna os parametros da query a ser utilizada pelo registro de cabecalho
     * @return string
     */
    public function obtemParametrosConsultaRegistro(): array {
        return [
            'P_NRORG' => $this->getParametros()->getNrorg(),
            'P_NRESTRUTURAM' => $this->getParametros()->getNrEstruturamEmpregador(),
            'P_NRPERIODOAPURACAO' => $this->getParametros()->getNrPeriodoApuracao(),
            'P_DTMESCOMPETENC' => $this->getParametros()->getDtMesCompetenc(),
            'P_INICIO' => $this->getParametros()->getDataInicio(),
            'P_FIM' => $this->getParametros()->getDataFim(),
            'P_NRORG_PADRAO' => $this->getParametros()->getNrorgPadrao(),
        ];
    }
}