<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\Util\Repositories;
use Folha\Entities\SstOcorrencia;
use Doctrine\ORM\EntityManager;
use Folha\Entities\SstVitima;
use HCM\Util\Messages;

class OcorrenciaController extends Crud {
    
    protected $ocorrenciaService;
    protected $dataSourceName = 'sst_ocorrencia_get';
    protected $dataSourceName1 = 'sst_vitima_get';
    protected $dataSourceName2 = 'sst_vitimaparteatingida_get';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\SST\Services\OcorrenciaService $ocorrenciaService,EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->ocorrenciaService = $ocorrenciaService;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    public function getOcorrencia(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setOcorrencia(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_ocorrencia";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtUmDiaUtilAposOcorrencia = date_create_from_format( 'd/m/Y g:h:i', explode(' ', $row['DTHROCORRENCIA'])[0].'00:00:00');
                do{
                    $dtUmDiaUtilAposOcorrencia->modify('+1 day');
                } while(!DateUtil::isDiaSemanaUtil(date_format($dtUmDiaUtilAposOcorrencia, 'd/m/Y')));
                $dtAtual = DateUtil::truncateData(DateUtil::getDataAtual(), 'd/m/Y');
                if(date_timestamp_get($dtUmDiaUtilAposOcorrencia) != date_timestamp_get($dtAtual)){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::AVISO_ESO_PENSALIDADE_REGISTRO_DATA, DTO\Response\Message::TYPE_ERROR));
                }
                if (array_key_exists('NROCORRENCIA', $row)) {
                    if ($row['NROCORRENCIA'] === '') {
                        $row['NROCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_OCORRENCIA', 1, 12, true);
                    }
                } else {
                    $row['NROCORRENCIA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_OCORRENCIA', 1, 12, true);
                }
                if(!array_key_exists('NRINFLUENCIAGERENCIAL', $row)){
                    $row['NRINFLUENCIAGERENCIAL'] = null;
                }
                if(!array_key_exists('NRACAO', $row)){
                    $row['NRACAO'] = null;
                }
                if($row['QTVITIMASOCORRE'] === null){
                    $row['QTVITIMASOCORRE'] = 0;
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getVitima(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setVitima(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_vitima";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if($row['__is_new'] === true){
                    $vitima = SstVitima::findOneByNrocorrenciaAndNrorgAndNrparcnegocioAndNrvitima($row['NROCORRENCIA'], $row['NRORG'], $row['NRPARCNEGOCIO'], $row['NRVITIMA']);
                    if(isset($vitima)){
                        return $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM05000_SST_REUNIAO_EXISTS_PARTICIPANTESCADASTRADO_ERROR, DTO\Response\Message::TYPE_ERROR));
                    }
                }    
                if (array_key_exists('NRVITIMA', $row)) {
                    if ($row['NRVITIMA'] === '') {
                        $row['NRVITIMA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_VITIMA', 1, 12, true);
                    }
                } else {
                    $row['NRVITIMA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_VITIMA', 1, 12, true);
                }
                if(!array_key_exists('NRINFLUENCIAGERENCIAL', $row)){
                    $row['NRINFLUENCIAGERENCIAL'] = null;
                }
                if(!array_key_exists('CDPAIS', $row)){
                    $row['CDPAIS'] = null;
                }
                if(!array_key_exists('SGESTADO', $row)){
                    $row['SGESTADO'] = null;
                }
                if(!array_key_exists('CDMUNICIPIO', $row)){
                    $row['CDMUNICIPIO'] = null;
                }
                if(!array_key_exists('CDLOGRADOURO', $row)){
                    $row['CDLOGRADOURO'] = null;
                }
                if(array_key_exists('NMBAIRROENDERECO', $row)){
                    $row['NMBAIRROENDERECO'] = Util::removeCaracteresEspeciais($row['NMBAIRROENDERECO']);
                }
                if(!array_key_exists('DSCOMPLEENDERECO', $row)){
                    $row['DSCOMPLEENDERECO'] = null;
                }
                if(!array_key_exists('NRCEPENDERECO', $row)){
                    $row['NRCEPENDERECO'] = null;
                } else {
                    $row['NRCEPENDERECO'] = str_replace(['-', '.'], "", $row['NRCEPENDERECO']);
                }
                if(!array_key_exists('DSREFERENCIAENDE', $row)){
                    $row['DSREFERENCIAENDE'] = null;
                }
                if(!array_key_exists('DSENDERECO', $row)){
                    $row['DSENDERECO'] = null;
                } else {
                    $row['DSENDERECO'] = Util::removeCaracteresEspeciais($row['DSENDERECO']);
                }
                if(!array_key_exists('NRIMOVELENDERECO', $row)){
                    $row['NRIMOVELENDERECO'] = null;
                }
                if(!array_key_exists('NRTIPOLOCALCAT', $row)){
                    $row['NRTIPOLOCALCAT'] = null;
                }if(!array_key_exists('NRAGENTCAUSAD', $row)){
                    $row['NRAGENTCAUSAD'] = null;
                }if(!array_key_exists('IDACIDFATAL', $row)){
                    $row['IDACIDFATAL'] = null;
                }
                
                
                if(!empty($row['DTHRINICATEND']) && !empty($row['DTHRFIMATEND'])){
                    $dataInicial = DateUtil::getDataDeString($row['DTHRINICATEND'],"d/m/Y H:i");
                    $dataFinal = DateUtil::getDataDeString($row['DTHRFIMATEND'],"d/m/Y H:i");
                    $intervalo = $dataInicial->diff($dataFinal);
                    $horas = $intervalo->h;
                    $minutos = $intervalo->i;
                    $dias = $intervalo->days;
                    $horas = $horas + (24 * $dias);
                    $horas = str_pad($horas,2,'0',STR_PAD_LEFT);
                    $minutos = str_pad($minutos,2,'0',STR_PAD_LEFT); 
                    $horasMinutos = $horas.$minutos;
                    $row['QTHRATENDIMENTO'] = $horasMinutos;
                    $response->addMessage( new DTO\Response\Message('QTHRATENDIMENTO_'.substr($horasMinutos, 0,  2).substr($horasMinutos, 2,  2), 'M') );
                }else{
                    $row['QTHRATENDIMENTO'] = null;
                    $response->addMessage( new DTO\Response\Message('QTHRATENDIMENTO_', 'M') );
                }
                if($row['__is_new'] === true){
                    $getOcorrencia = SstOcorrencia::findByNrocorrenciaAndNrorg($row['NROCORRENCIA'],$row['NRORG']);
                    foreach ($getOcorrencia as $row) {
                        $qtvitimas = $row->getQtvitimasocorre();
                        $qtvitimas += 1;
                        $row->setQtvitimasocorre($qtvitimas);
                        $this->entityManager->persist($row);
                        $this->entityManager->flush();
                    }
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getParteAtingidaOcorrencia(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setParteAtingidaOcorrencia(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_vitimaparteatingida";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRVITPARTEATING', $row)) {
                    if ($row['NRVITPARTEATING'] === '') {
                        $row['NRVITPARTEATING'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_VITIMAPARTEATINGIDA', 1, 12, true);
                    }
                } else {
                    $row['NRVITPARTEATING'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_VITIMAPARTEATINGIDA', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteParteAtingindaOcorrencia(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "sst_vitimaparteatingida";
        parent::delete($request, $response);
    }
    
    public function deleteOcorrencia(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->ocorrenciaService->deleteOcorrencia($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function deleteVitima(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->ocorrenciaService->deleteVitima($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
            $getOcorrencia = SstOcorrencia::findByNrocorrenciaAndNrorg($dataset['NROCORRENCIA'],$dataset['NRORG']);
            foreach ($getOcorrencia as $row) {
                $qtvitimas = $row->getQtvitimasocorre();
                $qtvitimas -= 1;
                $row->setQtvitimasocorre($qtvitimas);
                $this->entityManager->persist($row);
                $this->entityManager->flush();
            }
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 